@@ -65,7 +65,8 @@ def get_attribute_name_parts(node):
     return parts[::-1]
 
 
-def _anno_parser(node: ast.AST, imports: List[str], disable_from_import: bool) -> str:
+def _anno_parser(node: ast.AST, imports: List[str],
+                 disable_from_import: bool) -> str:
     if isinstance(node, ast.Module):
         assert len(node.body) == 1
         return _anno_parser(node.body[0], imports, disable_from_import)
@@ -76,17 +77,20 @@ def _anno_parser(node: ast.AST, imports: List[str], disable_from_import: bool) -
         if compat.Python3_9AndLater:
             # >= 3.9 ast.Index is deprecated
             assert isinstance(node.slice, ast.Tuple)
-            return "{}[{}]".format(_anno_parser(node.value, imports, disable_from_import),
-                                   _anno_parser(node.slice, imports, disable_from_import))
+            return "{}[{}]".format(
+                _anno_parser(node.value, imports, disable_from_import),
+                _anno_parser(node.slice, imports, disable_from_import))
         else:
             assert isinstance(node.slice, ast.Index)
             assert isinstance(
                 node.slice.value,
                 (ast.Tuple, ast.Attribute, ast.Name, ast.Subscript))
-            return "{}[{}]".format(_anno_parser(node.value, imports, disable_from_import),
-                                   _anno_parser(node.slice.value, imports, disable_from_import))
+            return "{}[{}]".format(
+                _anno_parser(node.value, imports, disable_from_import),
+                _anno_parser(node.slice.value, imports, disable_from_import))
     elif isinstance(node, ast.Tuple):
-        return ", ".join(_anno_parser(e, imports, disable_from_import) for e in node.elts)
+        return ", ".join(
+            _anno_parser(e, imports, disable_from_import) for e in node.elts)
     elif isinstance(node, ast.Name):
         return node.id
     elif isinstance(node, ast.Attribute):
@@ -96,8 +100,8 @@ def _anno_parser(node: ast.AST, imports: List[str], disable_from_import: bool) -
             imports.append(regular_import)
             return ".".join(value_parts)
         else:
-            from_import = "from {} import {}".format(".".join(value_parts[:-1]),
-                                                    value_parts[-1])
+            from_import = "from {} import {}".format(
+                ".".join(value_parts[:-1]), value_parts[-1])
             imports.append(from_import)
             return value_parts[-1]
     else:
@@ -310,7 +314,9 @@ class Pybind11MethodMeta(Pybind11Meta):
                  ret_policy: ReturnPolicy = ReturnPolicy.Auto,
                  call_guard: Optional[str] = None,
                  virtual: bool = False,
-                 keep_alives: Optional[List[Tuple[int, int]]] = None):
+                 keep_alives: Optional[List[Tuple[int, int]]] = None,
+                 is_raw_bind: bool = False,
+                 raw_bind_anno: str = ""):
         super().__init__(Pybind11SplitImpl)
         self.bind_name = bind_name
         self.method_type = method_type
@@ -319,6 +325,8 @@ class Pybind11MethodMeta(Pybind11Meta):
         self.call_guard = call_guard
         self.virtual = virtual
         self.keep_alives = keep_alives
+        self.is_raw_bind = is_raw_bind
+        self.raw_bind_anno = raw_bind_anno
 
 
 class Pybind11PropMeta(Pybind11Meta):
@@ -327,14 +335,21 @@ class Pybind11PropMeta(Pybind11Meta):
     def __init__(self, name: str, readwrite: bool = True):
         super().__init__(Pybind11SplitImpl)
         self.readwrite = readwrite
-        self.name = name 
+        self.name = name
 
+class Pybind11BindMethodMeta(Pybind11Meta):
+    """may add some attributes in future.
+    """
+    def __init__(self):
+        super().__init__(Pybind11SplitImpl)
 
 class PybindMethodDecl(object):
     def __init__(self, decl: FunctionDecl, namespace: str, class_name: str,
                  mw_meta: Pybind11MethodMeta):
         self.decl = decl
         self.mw_meta = mw_meta
+        if mw_meta.is_raw_bind:
+            assert isinstance(decl.meta, StaticMemberFunctionMeta), "bind method must be static"
         self.func_name = decl.get_function_name()
         self.bind_name = self.func_name
         if mw_meta.bind_name:
@@ -472,7 +487,7 @@ class PybindClassMixin:
         if mw_metas is None:
             mw_metas = []
         if prop_name is None:
-            prop_name = name 
+            prop_name = name
         name_part = name.split(",")
         prop_name_part = prop_name.split(",")
         assert len(name_part) == len(prop_name_part)
@@ -489,9 +504,13 @@ class PybindClassMixin:
                             mw_metas=mw_metas_part)
         return
 
-def _postprocess_class(cls_name: str, cls_namespace: str, submod: str,
+
+def _postprocess_class(cls_name: str,
+                       cls_namespace: str,
+                       submod: str,
                        decls: List[Union[PybindMethodDecl, PybindPropDecl]],
-                       enum_classes: List[EnumClass]):
+                       enum_classes: List[EnumClass],
+                       parent_ns: str = ""):
     has_virtual = False
     vblock = None  # Optional[Block]
     virtual_decls = []  # type: List[PybindMethodDecl]
@@ -499,7 +518,8 @@ def _postprocess_class(cls_name: str, cls_namespace: str, submod: str,
     prop_decls = []  # type: List[PybindPropDecl]
     for decl in decls:
         if isinstance(decl, PybindMethodDecl):
-            method_decls.append(decl)
+            if not decl.mw_meta.is_raw_bind:
+                method_decls.append(decl)
         else:
             prop_decls.append(decl)
     for decl in method_decls:
@@ -557,11 +577,15 @@ def _postprocess_class(cls_name: str, cls_namespace: str, submod: str,
             cls_namespace.replace(".", "::"), virtual_cls_name))
     cls_def_argu_str = ", ".join(cls_def_arguments)
     cls_def_name = "{}_{}".format(submod, cls_name)
-    cls_def = "pybind11::class_<{cls_def_argu_str}> {def_name}({submod}, \"{cls_name}\");".format(
+    parent = ""
+    if parent_ns:
+        parent = ", {}".format(parent_ns)
+    cls_def = "pybind11::class_<{cls_def_argu_str}{parent}> {def_name}({submod}, \"{cls_name}\");".format(
         cls_def_argu_str=cls_def_argu_str,
         def_name=cls_def_name,
         submod=submod,
-        cls_name=cls_name)
+        cls_name=cls_name,
+        parent=parent)
     cls_def_stmts = [cls_def]  # type: List[Union[Block, str]]
     if not has_constructor:
         cls_def_stmts.append(
@@ -694,7 +718,8 @@ def _generate_python_interface_class(cls_name: str,
     prop_decls = []  # type: List[PybindPropDecl]
     for decl in decls:
         if isinstance(decl, PybindMethodDecl):
-            method_decls.append(decl)
+            if not decl.mw_meta.is_raw_bind:
+                method_decls.append(decl)
         else:
             prop_decls.append(decl)
     name_to_overloaded = OrderedDict(
@@ -714,8 +739,8 @@ def _generate_python_interface_class(cls_name: str,
         default_str = ""
         if default is not None:
             default_str = " = {}".format(default)
-        decl_codes.append("{}: {}{}".format(prop_decl.get_prop_name(), prop_anno,
-                                            default_str))
+        decl_codes.append("{}: {}{}".format(prop_decl.get_prop_name(),
+                                            prop_anno, default_str))
     for decl in method_decls:
         if decl.bind_name not in name_to_overloaded:
             name_to_overloaded[decl.bind_name] = []
@@ -827,6 +852,7 @@ class Pybind11SingleClassHandler(ManualClass):
         self.cu = cu
 
         self.bind_func_name = "bind_{}".format(cu.class_name)
+        self.raw_bind_annos: List[str] = []
 
     def get_pybind_decls(
             self) -> List[Union[PybindMethodDecl, PybindPropDecl]]:
@@ -838,30 +864,39 @@ class Pybind11SingleClassHandler(ManualClass):
     def handle_function_decl(self, cu: Class, func_decl: FunctionDecl,
                              mw_meta: Pybind11MethodMeta):
         assert cu.namespace is not None
+        if mw_meta.raw_bind_anno:
+            self.raw_bind_annos.append(mw_meta.raw_bind_anno)
         self.func_decls.append(
             PybindMethodDecl(func_decl, cu.namespace, cu.class_name, mw_meta))
 
     def handle_member(self, cu: Class, member_decl: Member,
                       mw_meta: Pybind11PropMeta):
         assert cu.namespace is not None
+
         self.prop_decls.append(
             PybindPropDecl(member_decl, cu.namespace, cu.class_name, mw_meta))
 
-    def postprocess(self):
+    def postprocess(self, parent_is_pybind: bool = False):
         if self.built:
             return
         bind_code = FunctionCode()
         bind_code.arg("module", "const pybind11::module_&")
         func_meta = StaticMemberFunctionMeta(impl_file_suffix=self.file_suffix)
+        parent_name = "" if not parent_is_pybind else self.cu.get_parent_name()
         cls_def_block, vblock = _postprocess_class(self.cu.class_name,
                                                    self.cu.namespace, "module",
                                                    self.get_pybind_decls(),
-                                                   self.cu._enum_classes)
+                                                   self.cu._enum_classes,
+                                                   parent_name)
         func_meta.name = self.bind_func_name
         if vblock is not None:
             bind_code.code_after_include = "\n".join(
                 generate_code(vblock, 0, 2))
         bind_code.raw("\n".join(generate_code(cls_def_block, 0, 2)))
+        for decl in self.func_decls:
+            if decl.mw_meta.is_raw_bind:
+                bind_code.raw("{}::{}(module);".format(self.cu.canonical_name,
+                                                    decl.func_name))
         func_decl = FunctionDecl(func_meta, bind_code)
         self.add_func_decl(func_decl)
         # TODO better code
@@ -871,6 +906,8 @@ class Pybind11SingleClassHandler(ManualClass):
         else:
             self.add_impl_only_param_class_by_name(self.bind_func_name, "bind",
                                                    self.cu)
+
+
         self.built = True
 
 
@@ -914,12 +951,14 @@ class Pybind11SplitMain(ParameterizedClass):
             assert bind_cu.namespace is not None
             bind_ns = bind_cu.namespace
             submodule_name = submodules[bind_ns]
-            bind_func_name = "{}::{}::{}".format(bind_ns.replace(".", "::"),
+            bind_cpp_ns = bind_ns.replace(".", "::")
+            bind_func_name = "{}::{}::{}".format(bind_cpp_ns,
                                                  bind_cu.class_name,
                                                  bind_cu.bind_func_name)
             create_stmts.append("{}({});".format(bind_func_name,
                                                  submodule_name))
 
+
         code_block = Block("PYBIND11_MODULE({}, m){{".format(self.module_name),
                            sub_defs + create_stmts, "}")
         code = generate_code(code_block, 0, 2)
@@ -940,10 +979,11 @@ class Pybind11SplitMain(ParameterizedClass):
         """
         init_import = "from typing import overload, Any, Callable, Dict, List, Optional, Set, Tuple, Type, Union"
         init_pccm_import = "from pccm.stubs import EnumValue, EnumClassValue"
-
         ns_to_interfaces = OrderedDict()  # type: Dict[str, List[Block]]
         ns_to_imports = OrderedDict()  # type: Dict[str, List[str]]
         ns_to_interface = OrderedDict()  # type: Dict[str, str]
+        ns_to_raw_bind_annos = OrderedDict()  # type: Dict[str, List[str]]
+
         exist_annos = {}  # type: Dict[str, str]
         for bind_cu in bind_cus:
             exist_annos.update(_collect_exist_annos(
@@ -955,11 +995,13 @@ class Pybind11SplitMain(ParameterizedClass):
             if ns not in ns_to_interfaces:
                 ns_to_interfaces[ns] = []
                 ns_to_imports[ns] = []
+                ns_to_raw_bind_annos[ns] = []
             class_block, cls_imports = _generate_python_interface_class(
                 origin_cu.class_name, bind_cu.get_pybind_decls(),
                 origin_cu._enum_classes, exist_annos)
             ns_to_imports[ns].extend(cls_imports)
             ns_to_interfaces[ns].append(class_block)
+            ns_to_raw_bind_annos[ns].extend(bind_cu.raw_bind_annos)
         module_as_init = set()  # type: Set[str]
         for k, interfaces in ns_to_interfaces.items():
             k_prefix = ".".join(k.split(".")[:-1])
@@ -979,6 +1021,8 @@ class Pybind11SplitMain(ParameterizedClass):
             imports = _unique_list_keep_order(imports)
             ns_to_interface[k_file] = "\n".join(
                 generate_code_list(imports + interfaces, 0, 4))
+            if k in ns_to_raw_bind_annos:
+                ns_to_interface[k_file] += "\n" + "\n".join(ns_to_raw_bind_annos[k])
         return ns_to_interface
 
 
@@ -1005,8 +1049,12 @@ class Pybind11SplitImpl(ManualClassGenerator):
         return bind_cu
 
     def get_code_units(self) -> List[Class]:
+        bind_cu_ids: Set[str] = set()
         for bind in self.bind_cus:
-            bind.postprocess()
+            bind_cu_ids.add(bind.cu.canonical_name)
+        for bind in self.bind_cus:
+            parent_name = bind.cu.get_parent_name()
+            bind.postprocess(parent_name in bind_cu_ids)
         self.main_cu.postprocess(self.bind_cus)
         res = []  # type: List[Class]
         res.extend(self.bind_cus)
@@ -1024,7 +1072,9 @@ def mark(func=None,
          ret_policy: ReturnPolicy = ReturnPolicy.Auto,
          virtual: bool = False,
          nogil: bool = False,
-         keep_alives: Optional[List[Tuple[int, int]]] = None):
+         keep_alives: Optional[List[Tuple[int, int]]] = None,
+         _is_raw_bind: bool = False,
+         _raw_bind_anno: str = ""):
     if virtual:
         assert not nogil, "you can't release gil for python virtual function."
     call_guard = None  # type: Optional[str]
@@ -1032,10 +1082,14 @@ def mark(func=None,
         call_guard = "pybind11::gil_scoped_release"
     pybind_meta = Pybind11MethodMeta(bind_name, method_type, prop_name,
                                      ret_policy, call_guard, virtual,
-                                     keep_alives)
+                                     keep_alives, _is_raw_bind, _raw_bind_anno)
     return middleware_decorator(func, pybind_meta)
 
 
+def mark_bind_raw(func=None, raw_bind_anno: str = ""):
+    return mark(func, _is_raw_bind=True, _raw_bind_anno=raw_bind_anno)
+
+
 def mark_prop_getter(func=None,
                      prop_name: str = "",
                      ret_policy: ReturnPolicy = ReturnPolicy.Auto,
